@@ -1,4 +1,4 @@
-/* Proxytunnel - (C) 2001-2008 Jos Visser / Mark Janssen    */
+/* Proxytunnel - (C) 2001-2020 Jos Visser / Mark Janssen    */
 /* Contact:                  josv@osp.nl / maniac@maniac.nl */
 
 /*
@@ -38,30 +38,41 @@ extern char * optarg;
 static char *getCredentialsFromFile( const char* filename, char **user, char **pass, char **rem_user, char **rem_pass);
 
 void cmdline_parser_print_version (void) {
-	printf ("%s %s (rev %d) Copyright 2001-2008 Proxytunnel Project\n", PACKAGE, VERSION, REV);
+	printf ("%s %s Copyright 2001-2020 Proxytunnel Project\n", PACKAGE, VERSION);
 }
 
 void cmdline_parser_print_help (void) {
 	cmdline_parser_print_version ();
 	printf(
 "Usage: %s [OPTIONS]...\n"
-"Build generic tunnels trough HTTPS proxy's, supports HTTP authorization\n"
+"Build generic tunnels through HTTPS proxies using HTTP authentication\n"
 "\n"
 "Standard options:\n"
 // FIXME: "   -c, --config=FILE       Read config options from file\n"
-" -i, --inetd               Run from inetd (default=off)\n"
+" -i, --inetd               Run from inetd (default: off)\n"
 " -a, --standalone=INT      Run as standalone daemon on specified port\n"
-// FIXME: " -f, --nobackground      Don't for to background in standalone mode\n"
+// FIXME: " -f, --nobackground        Don't for tok background in standalone mode\n"
 " -p, --proxy=STRING        Local proxy host:port combination\n"
 " -r, --remproxy=STRING     Remote proxy host:port combination (using 2 proxies)\n"
 " -d, --dest=STRING         Destination host:port combination\n"
 #ifdef USE_SSL
 " -e, --encrypt             SSL encrypt data between local proxy and destination\n"
 " -E, --encrypt-proxy       SSL encrypt data between client and local proxy\n"
-" -X, --encrypt-remproxy    Encrypt between 1st and 2nd proxy using SSL\n"
+" -X, --encrypt-remproxy    SSL encrypt data between local and remote proxy\n"
+" -W, --wa-bug-29744        Workaround ASF Bugzilla 29744: if SSL is active stop\n"
+"                           using it after CONNECT (might not work on all setups; see\n"
+"                           /usr/share/doc/proxytunnel/README.Debian.gz)\n"
+" -B, --buggy-encrypt-proxy Equivalent to -E -W, provided for backwards\n"
+"                           compatibility\n"
+" -L                        (legacy) enforce TLSv1 connection\n"
+" -T, --no-ssl3             Do not connect using SSLv3\n"
 #endif
 "\n"
 "Additional options for specific features:\n"
+#ifdef USE_SSL
+" -z, --no-check-certficate Don't verify server SSL certificate\n"
+" -C, --cacert=STRING       Path to trusted CA certificate or directory\n"
+#endif
 " -F, --passfile=STRING     File with credentials for proxy authentication\n"
 " -P, --proxyauth=STRING    Proxy auth credentials user:pass combination\n"
 " -R, --remproxyauth=STRING Remote proxy auth credentials user:pass combination\n"
@@ -72,6 +83,7 @@ void cmdline_parser_print_help (void) {
 " -N, --ntlm                Use NTLM based authentication\n"
 " -t, --domain=STRING       NTLM domain (default: autodetect)\n"
 " -H, --header=STRING       Add additional HTTP headers to send to proxy\n"
+" -o STRING                 send custom Host Header\n"
 #ifdef SETPROCTITLE
 " -x, --proctitle=STRING    Use a different process title\n"
 #endif
@@ -131,7 +143,11 @@ int cmdline_parser( int argc, char * const *argv, struct gengetopt_args_info *ar
 	args_info->encrypt_given = 0;
 	args_info->encryptproxy_given = 0;
 	args_info->encryptremproxy_given = 0;
+	args_info->wa_bug_29744_given = 0;
 	args_info->proctitle_given = 0;
+	args_info->enforcetls1_given = 0;
+	args_info->host_given = 0;
+	args_info->cacert_given = 0;
 
 /* No... we can't make this a function... -- Maniac */
 #define clear_args() \
@@ -156,8 +172,14 @@ int cmdline_parser( int argc, char * const *argv, struct gengetopt_args_info *ar
 	args_info->encrypt_flag = 0; \
 	args_info->encryptproxy_flag = 0; \
 	args_info->encryptremproxy_flag = 0; \
+	args_info->wa_bug_29744_flag = 0; \
+	args_info->no_ssl3_flag = 0; \
 	args_info->proctitle_arg = NULL; \
-} 
+	args_info->enforcetls1_flag = 0; \
+	args_info->host_arg = NULL; \
+	args_info->no_check_cert_flag = 0; \
+	args_info->cacert_arg = NULL; \
+}
 
 	clear_args();
 
@@ -189,6 +211,8 @@ int cmdline_parser( int argc, char * const *argv, struct gengetopt_args_info *ar
 			{ "remproxy",		1, NULL, 'r' },
 			{ "remproxyauth",	1, NULL, 'R' },
 			{ "proctitle",		1, NULL, 'x' },
+			{ "host",           1, NULL, 'o' },
+			{ "tlsenforce",     1, NULL, 'L' },
 			{ "header",			1, NULL, 'H' },
 			{ "verbose",		0, NULL, 'v' },
 			{ "ntlm",			0, NULL, 'N' },
@@ -198,12 +222,17 @@ int cmdline_parser( int argc, char * const *argv, struct gengetopt_args_info *ar
 			{ "encrypt",		0, NULL, 'e' },
 			{ "encrypt-proxy",	0, NULL, 'E' },
 			{ "encrypt-remproxy",0,NULL, 'X' },
+			{ "wa-bug-29744",	0, NULL, 'W' },
+			{ "buggy-encrypt-proxy",	0, NULL, 'B' },
+			{ "no-ssl3",		0, NULL, 'T' },
+			{ "no-check-certificate",0,NULL,'z' },
+			{ "cacert",         1, NULL, 'C' },
 			{ NULL,				0, NULL, 0 }
 		};
 
-		c = getopt_long (argc, argv, "hVia:u:s:t:F:p:P:r:R:d:H:x:nvNeEXq", long_options, &option_index);
+		c = getopt_long (argc, argv, "hVia:u:s:t:F:p:P:r:R:d:H:x:nvNeEXWBqLo:TzC:", long_options, &option_index);
 #else
-		c = getopt( argc, argv, "hVia:u:s:t:F:p:P:r:R:d:H:x:nvNeEXq" );
+		c = getopt( argc, argv, "hVia:u:s:t:F:p:P:r:R:d:H:x:nvNeEXWBqLo:TzC:" );
 #endif
 
 		if (c == -1)
@@ -227,6 +256,19 @@ int cmdline_parser( int argc, char * const *argv, struct gengetopt_args_info *ar
 				if( args_info->verbose_flag )
 					message("SSL client to proxy enabled\n");
 				break;
+
+			case 'W':	/* if SSL is active stop it after CONNECT */
+				args_info->wa_bug_29744_flag = !(args_info->wa_bug_29744_flag);
+				if( args_info->verbose_flag )
+					message("If SSL is active stop it after CONNECT\n");
+				break;
+
+			case 'B':	/* do -E -W */
+				args_info->wa_bug_29744_flag = !(args_info->wa_bug_29744_flag);
+				args_info->encryptproxy_flag = !(args_info->encryptproxy_flag);
+				if( args_info->verbose_flag )
+					message("SSL client to proxy enabled, only until CONNECT\n");
+				break;
 #endif
 
 			case 'i':	/* Run from inetd. */
@@ -262,6 +304,18 @@ int cmdline_parser( int argc, char * const *argv, struct gengetopt_args_info *ar
 				args_info->proctitle_arg = gengetopt_strdup (optarg);
 				break;
 
+			case 'L':
+				args_info->enforcetls1_given = 1;
+				message("Enforcing TLSv1");
+				args_info->enforcetls1_flag = 1;
+				break;
+
+			case 'o':
+				args_info->host_given = 1;
+				message("Host-header override enabled\n");
+				args_info->host_arg = gengetopt_strdup (optarg);
+				break;
+
 			case 'u':	/* Username to send to HTTPS proxy for authentication.  */
 				if (args_info->user_given) {
 					fprintf (stderr, "%s: `--user' (`-u'), `--proxyauth' (`-P') or `--passfile' (`-F') option given more than once\n", PACKAGE);
@@ -370,6 +424,11 @@ int cmdline_parser( int argc, char * const *argv, struct gengetopt_args_info *ar
 					message("SSL local to remote proxy enabled\n");
 				break;
 
+			case 'T':   /* Turn off SSLv3 */
+				args_info->no_ssl3_flag = !(args_info->no_ssl3_flag);
+				if( args_info->verbose_flag )
+					message("SSLv3 disabled\n");
+				break;
 
 			case 'd':	/* Destination host to built the tunnel to.  */
 				if (args_info->dest_given) {
@@ -403,6 +462,22 @@ int cmdline_parser( int argc, char * const *argv, struct gengetopt_args_info *ar
 				args_info->quiet_flag = !(args_info->quiet_flag);
 				break;
 
+			case 'z':	/* Don't verify server SSL certificate */
+				args_info->no_check_cert_flag = 1;
+				if( args_info->verbose_flag )
+					message("Server SSL certificate verification disabled\n");
+				break;
+
+			case 'C':	/* Trusted CA certificate (or directory) for server SSL certificate verification */
+				if (args_info->cacert_given) {
+					fprintf (stderr, "%s: `--cacert' (`-C') option given more than once\n", PACKAGE);
+					clear_args ();
+					exit(1);
+				}
+				args_info->cacert_given = 1;
+				args_info->cacert_arg = gengetopt_strdup (optarg);
+				break;
+
 			case 0:	/* Long option with no short option */
 
 			case '?':	/* Invalid option.  */
@@ -460,11 +535,12 @@ int cmdline_parser( int argc, char * const *argv, struct gengetopt_args_info *ar
 	}
 
 	if ( args_info->proxy_arg == NULL ) {
-		if ( (tmp = getenv("HTTP_PROXY")) != NULL ) {
-			int r;
+		if ( ((tmp = getenv("http_proxy")) != NULL) || ((tmp = getenv("HTTP_PROXY")) != NULL) ) {
+			//int r;
 			char * temp;
 			temp = malloc( 56+1 );
-			r = sscanf( tmp, "http://%56[^/]/", temp );
+			sscanf( tmp, "http://%56[^/]/", temp );
+			//r = sscanf( tmp, "http://%56[^/]/", temp );
 //			message( "r = '%d'\ntemp = '%s'\n", r, temp);
 
 			args_info->proxy_given = 1;
@@ -474,7 +550,7 @@ int cmdline_parser( int argc, char * const *argv, struct gengetopt_args_info *ar
 		}
 	}
 
-	if (! args_info->proxy_given || ! args_info->dest_given ) {
+	if (! args_info->proxy_given && ! args_info->dest_given ) {
 		clear_args ();
 //		cmdline_parser_print_help ();
 		message( "No proxy or destination given, exiting\nUse '--help' flag for usage info\n" );
@@ -482,19 +558,30 @@ int cmdline_parser( int argc, char * const *argv, struct gengetopt_args_info *ar
 	}
 
 	if (args_info->proxy_given ) {
+		char proxy_arg_fmt[32];
+		size_t proxy_arg_len;
 		char * phost;
 		int pport;
 
-		phost = malloc( 50+1 );
-
-		r = sscanf( args_info->proxy_arg, "%50[^:]:%5u", phost, &pport );
+		proxy_arg_len = strlen( args_info->proxy_arg );
+		if ( (phost = malloc( proxy_arg_len + 1 )) == NULL ) {
+			message( "Out of memory\n" );
+			exit(1);
+		}
+		snprintf( proxy_arg_fmt, sizeof(proxy_arg_fmt), "%%%zu[^:]:%%5u", proxy_arg_len - 1 );
+		r = sscanf( args_info->proxy_arg, proxy_arg_fmt, phost, &pport );
+		if ( r != 2 ) {
+			/* try bracket-enclosed IPv6 literal */
+			snprintf( proxy_arg_fmt, sizeof(proxy_arg_fmt), "[%%%zu[^]]]:%%5u", proxy_arg_len - 1 );
+			r = sscanf( args_info->proxy_arg, proxy_arg_fmt, phost, &pport );
+		}
 		if ( r == 2 ) {
 			args_info->proxyhost_arg = phost;
 			args_info->proxyport_arg = pport;
 			args_info->proxyhost_given = 1;
 			args_info->proxyport_given = 1;
 		} else {
-			message( "parse_cmdline: couln't find your proxy hostname/ip (%s)\n", args_info->proxy_arg );
+			message( "parse_cmdline: could not find your proxy hostname/ip (%s)\n", args_info->proxy_arg );
 			missing_required_options++;
 		}
 	}
@@ -517,7 +604,7 @@ int cmdline_parser( int argc, char * const *argv, struct gengetopt_args_info *ar
 			args_info->user_arg = args_info->proxyauth_arg;
 			args_info->user_given = 1;
 		} else {
-			message( "parse_cmdline: couln't find your proxy auth user/pass\n" );
+			message( "parse_cmdline: could not find your proxy auth user/pass\n" );
 			missing_required_options++;
 		}
 	}
@@ -540,7 +627,7 @@ int cmdline_parser( int argc, char * const *argv, struct gengetopt_args_info *ar
 			args_info->remuser_arg = args_info->remproxyauth_arg;
 			args_info->remuser_given = 1;
 		} else {
-			message( "parse_cmdline: couln't find your proxy auth user/pass\n" );
+			message( "parse_cmdline: could not find your proxy auth user/pass\n" );
 			missing_required_options++;
 		}
 	}
